/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: gps-hw.c
 * Description: USART buffered receiver for the GPS module
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2009-03-25 13:48:21 +0200 (Wed, 25 Mar 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: gps-hw.c 658 2009-03-25 11:48:21Z dimitar $
 *
 ****************************************************************************/

#include "stm32f10x_lib.h"
#include "platform_config.h"

#include <assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

#include "gps-hw.h"
#include "utils.h"


/**
 * @file
 * USART buffered receiver for the GPS module
 *
 * Buffered receiver of GPS messages.
 *
 */

static void gps_hw_gpio_init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Configure USART2 Rx (PA.3) as input floating */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

static void gps_hw_enable_int(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;

	/* Enable the USART2 Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	/* Enable USART2 Receive interrupts */
	USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
}


static void gps_hw_init_usart(void)
{
	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 19200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx;
	/* Configure USART2 */
	USART_Init(USART2, &USART_InitStructure);

}


void gps_hw_init(void)
{
	gps_hw_gpio_init();
	gps_hw_init_usart();
	gps_hw_enable_int();

	/* Enable the USART2 */
	USART_Cmd(USART2, ENABLE);
}

/*-------------------------------------------------------------------------*/

#define LINE_SIZE	128
#define NUM_LINES	8
static char lines[NUM_LINES][LINE_SIZE];
static int current_line;
static int current_char_i;
static volatile int num_filled_lines;
  
/*******************************************************************************
* Function Name  : USART2_IRQHandler
* Description    : This function handles USART2 global interrupt request.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void USART2_IRQHandler(void)
{
	int c;

	if(USART_GetITStatus(USART2, USART_IT_RXNE) != RESET)
	{
		/* Read one byte from the receive data register */
		c = USART_ReceiveData(USART2); 

		if(c == '\n') {
			assert(current_char_i < LINE_SIZE);
			lines[current_line][current_char_i] = '\0';
			if(num_filled_lines < NUM_LINES) {
				num_filled_lines++;
				current_char_i = 0;
				current_line = (current_line+1) % NUM_LINES;
			}
		} else if((current_char_i < (LINE_SIZE-1)) && (c!='\r')) {
			lines[current_line][current_char_i] = c;
			current_char_i++;
		}
	}
}


char *gps_hw_get_line(void)
{
	char *p;

	enter_critical_section();
	if(num_filled_lines>0) {
		unsigned int i;

		i = ((unsigned int)current_line - (unsigned int)num_filled_lines);
		i = i%NUM_LINES;
		
		p = &lines[i][0];
	} else {
		p = 0;
	}
	exit_critical_section();
	return p;
}


void gps_hw_dealloc_line(void)
{
	enter_critical_section();
	assert(num_filled_lines>0);
	num_filled_lines--;
	exit_critical_section();
}

