/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: lcd-320x240_9341.c
 * Description: Driver for TFT display with ILI9341 controller
 *
 ****************************************************************************/

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include "lcd-320x240_9341.h"
#include "fonts.h"


/* AHB clock period in nanoseconds */
#define T_HCK	14



// data/command bit is index 20 - so data is | (1<<20)
#define LCDBUS_RSLOW_ADDR	((vu16 *)0x60000000)
#define LCDBUS_RSHIGH_ADDR	((vu16 *)0x60100000)

static void Delay(volatile int i)
{
  for(;i;--i)
  {
    volatile int j;
    for(j=0; j<100;++j);
  }
}

static void lcdreset_set(int val)
{
  if(val)
    GPIO_SetBits(GPIOE, GPIO_Pin_2);
  else
    GPIO_ResetBits(GPIOE, GPIO_Pin_2);
}

static void lcdled_set(int val)
{
  if(!val)
    GPIO_SetBits(GPIOD, GPIO_Pin_13);
  else
    GPIO_ResetBits(GPIOD, GPIO_Pin_13);
}

static void lcdreset_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  lcdreset_set(1);
}

static void lcdled_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  lcdled_set(1);
}


/* Initialize the FSMC bus used for driving the LCD */
static void FSMC_LCDBUS_Init(void)
{
  FSMC_NORSRAMInitTypeDef  FSMC_NORSRAMInitStructure;
  FSMC_NORSRAMTimingInitTypeDef  pw,pr;
  GPIO_InitTypeDef GPIO_InitStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOD | RCC_APB2Periph_GPIOG | RCC_APB2Periph_GPIOE |
                         RCC_APB2Periph_GPIOF, ENABLE);

/*-- GPIO Configuration ------------------------------------------------------*/
  /* SRAM Data lines configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_8 | GPIO_Pin_9 |
                                GPIO_Pin_10 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
                                GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
                                GPIO_Pin_15;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  //TODO - get rid of address lines!!!
  /* SRAM Address lines configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
                                GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_12 | GPIO_Pin_13 |
                                GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_Init(GPIOF, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
                                GPIO_Pin_4 | GPIO_Pin_5;
  GPIO_Init(GPIOG, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

  /* NOE and NWE configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 |GPIO_Pin_5;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  /* NE1 configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
  GPIO_Init(GPIOD, &GPIO_InitStructure);


  /* NBL0, NBL1 configuration */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

/*-- FSMC Configuration ------------------------------------------------------*/
  pr.FSMC_AddressSetupTime = (5/T_HCK+1);
  pr.FSMC_AddressHoldTime = (5/T_HCK+1);
  pr.FSMC_DataSetupTime = (100/T_HCK+1);
  pr.FSMC_BusTurnAroundDuration = 0;
  pr.FSMC_CLKDivision = 0;
  pr.FSMC_DataLatency = 0;
  pr.FSMC_AccessMode = FSMC_AccessMode_A;

  pw.FSMC_AddressSetupTime = (5/T_HCK+1);
  pw.FSMC_AddressHoldTime = (5/T_HCK+1);
  pw.FSMC_DataSetupTime = ((20+15)/T_HCK+1);
  pw.FSMC_BusTurnAroundDuration = 0;
  pw.FSMC_CLKDivision = 0;
  pw.FSMC_DataLatency = 0;
  pw.FSMC_AccessMode = FSMC_AccessMode_A;

  FSMC_NORSRAMInitStructure.FSMC_Bank = FSMC_Bank1_NORSRAM2;
  FSMC_NORSRAMInitStructure.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
  FSMC_NORSRAMInitStructure.FSMC_MemoryType = FSMC_MemoryType_SRAM;
  FSMC_NORSRAMInitStructure.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
  FSMC_NORSRAMInitStructure.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalPolarity = FSMC_WaitSignalPolarity_Low;
  FSMC_NORSRAMInitStructure.FSMC_WrapMode = FSMC_WrapMode_Disable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
  FSMC_NORSRAMInitStructure.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
  FSMC_NORSRAMInitStructure.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
  FSMC_NORSRAMInitStructure.FSMC_ExtendedMode = FSMC_ExtendedMode_Enable;
  FSMC_NORSRAMInitStructure.FSMC_WriteBurst = FSMC_WriteBurst_Enable;
  FSMC_NORSRAMInitStructure.FSMC_ReadWriteTimingStruct = &pr;
  FSMC_NORSRAMInitStructure.FSMC_WriteTimingStruct = &pw;

  FSMC_NORSRAMInit(&FSMC_NORSRAMInitStructure);

  /* Enable FSMC Bank1_SRAM Bank */
  FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM2, ENABLE);
}

static void LCDBUS_Send (int CmdData, u16 Value)
{
  if (CmdData == COMMAND)
    *LCDBUS_RSLOW_ADDR = Value;
  if (CmdData == DATA)
    *LCDBUS_RSHIGH_ADDR = Value;
}

static int LCDBUS_Read ()
{
  return *LCDBUS_RSHIGH_ADDR;
}

static void LCDBUS_WriteReg(int CommandCode, int Value)
{
  LCDBUS_Send (COMMAND, CommandCode);
  LCDBUS_Send (DATA, Value);
}

static int LCDBUS_ReadReg(int CommandCode)
{
  LCDBUS_Send (COMMAND, CommandCode);
  return LCDBUS_Read ();
}

// x=[0-240] , y=[0-320]
void LCD_SetPixel(unsigned int x, unsigned int y, u16 color)
{
  LCDBUS_Send (COMMAND, CMD_COLUMN_ADDRESS_SET); // Column address set
  LCDBUS_Send (DATA, x>>8);
  LCDBUS_Send (DATA, x);
  LCDBUS_Send (COMMAND, CMD_PAGE_ADDRESS_SET); // Page address set
  LCDBUS_Send (DATA, y>>8);
  LCDBUS_Send (DATA, y);

  LCDBUS_Send (COMMAND, CMD_MEMORY_WRITE); // memory write
  LCDBUS_Send (DATA, color);
}

void LCD_Init(void)
{
  FSMC_LCDBUS_Init();
  lcdreset_init();
  lcdled_init();
  lcdled_set(1);

  lcdreset_set(1);
  Delay(1000);
  lcdreset_set(0);
  Delay(1000);
  lcdreset_set(1);
  Delay(1000);
  
  Delay(500); /* delay 50 ms */
  
  //Start initial Sequence
  LCDBUS_Send(COMMAND, CMD_SOFTWARE_RESET); //software reset
  Delay(50);
  LCDBUS_Send(COMMAND, CMD_DISPLAY_OFF); // display off
  Delay(50);
  //-------power control------------------------------
  LCDBUS_Send(COMMAND, CMD_POWER1); //power control
  LCDBUS_Send(DATA, 0x26);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_POWER2); //power control
  LCDBUS_Send(DATA, 0x11);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_VCOM1); //vcom control
  LCDBUS_Send(DATA, 0x5c);//35
  LCDBUS_Send(DATA, 0x4c);//3E
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_VCOM2); //vcom control
  LCDBUS_Send(DATA, 0x94);
  Delay(50);
  //-------memory access control------------------------
  LCDBUS_Send(COMMAND, CMD_MEMORY_ACCESS_CONTROL); // memory access control
  LCDBUS_Send(DATA, 0x60); //0060 my,mx,mv,ml,RGB,mh,0.0
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_COLOR_PIXEL_FORMAT); // pixel format set
  LCDBUS_Send(DATA, 0x55);//16bit /pixel
  Delay(50);
  //------------ frame rate------------------------------
  LCDBUS_Send(COMMAND, CMD_FRAME_RATE_NORMAL); // frame rate
  LCDBUS_Send(DATA, 0x00);
  LCDBUS_Send(DATA, 0x1B); //70
  Delay(50);
  //-----------Gamma---------------------------------
  LCDBUS_Send(COMMAND, 0xf2); // 3Gamma Function Disable
  LCDBUS_Send(DATA, 0x08);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_GAMMA_SET); // gamma set 4 gamma curve 01/02/04/08
  LCDBUS_Send(DATA, 0x01); 
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_POSITIVE_GAMMA); //positive gamma correction
  LCDBUS_Send(DATA, 0x1f);
  LCDBUS_Send(DATA, 0x1a);
  LCDBUS_Send(DATA, 0x18);
  LCDBUS_Send(DATA, 0x0a);
  LCDBUS_Send(DATA, 0x0f);
  LCDBUS_Send(DATA, 0x06);
  LCDBUS_Send(DATA, 0x45);
  LCDBUS_Send(DATA, 0x87);
  LCDBUS_Send(DATA, 0x32);
  LCDBUS_Send(DATA, 0x0a);
  LCDBUS_Send(DATA, 0x07);
  LCDBUS_Send(DATA, 0x02);
  LCDBUS_Send(DATA, 0x07);
  LCDBUS_Send(DATA, 0x05);
  LCDBUS_Send(DATA, 0x00);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_NEGATIVE_GAMMA); //negative gamma correction
  LCDBUS_Send(DATA, 0x00);
  LCDBUS_Send(DATA, 0x25);
  LCDBUS_Send(DATA, 0x27);
  LCDBUS_Send(DATA, 0x05);
  LCDBUS_Send(DATA, 0x10);
  LCDBUS_Send(DATA, 0x09);
  LCDBUS_Send(DATA, 0x3a);
  LCDBUS_Send(DATA, 0x78);
  LCDBUS_Send(DATA, 0x4d);
  LCDBUS_Send(DATA, 0x05);
  LCDBUS_Send(DATA, 0x18);
  LCDBUS_Send(DATA, 0x0d);
  LCDBUS_Send(DATA, 0x38);
  LCDBUS_Send(DATA, 0x3a);
  LCDBUS_Send(DATA, 0x1f);
  Delay(50);
  
  //---------ddram ---------------------
  LCDBUS_Send(COMMAND, CMD_COLUMN_ADDRESS_SET); // column set
  LCDBUS_Send(DATA, 0x00);
  LCDBUS_Send(DATA, 0xEF);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_PAGE_ADDRESS_SET); // page address set
  LCDBUS_Send(DATA, 0x01);
  LCDBUS_Send(DATA, 0x3F);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_ENTRY_MODE); //entry mode set
  LCDBUS_Send(DATA, 0x07);
  Delay(50);
  //------------display---------------------
  LCDBUS_Send(COMMAND, CMD_DISPLAY_FUNCTION); // display function control
  LCDBUS_Send(DATA, 0x0a);
  LCDBUS_Send(DATA, 0x82);
  LCDBUS_Send(DATA, 0x27);
  LCDBUS_Send(DATA, 0x00);
  Delay(50);
  
  LCDBUS_Send(COMMAND, CMD_SLEEP_OUT); //sleep out
  Delay(100);
  LCDBUS_Send(COMMAND, CMD_DISPLAY_ON); // display on
  Delay(100);
  LCDBUS_Send(COMMAND, CMD_MEMORY_WRITE); //memory write
  Delay(200);
}

int LCD_ReadID ()
{
  unsigned int Data[4], i, ID;
  LCDBUS_Send (COMMAND, CMD_READ_ID4);
  for (i=0; i<4; i++)
    Data[i] = LCDBUS_Read();
  ID = (Data[2]<<8) | Data[3];
  return ID;
}

void LCD_ExitSleep(void)
{
  LCDBUS_Send(COMMAND, CMD_SLEEP_OUT); //standby out
  Delay(60);
  LCDBUS_Send(COMMAND, CMD_DISPLAY_ON);//display on
}


void LCD_EnterSleep(void)
{
  LCDBUS_Send(COMMAND, CMD_DISPLAY_OFF); //display off
  Delay (60);
  LCDBUS_Send(COMMAND, CMD_ENTER_SLEEP_MODE);//enter sleep  mode
  Delay (60);
}


void LCD_EnterDeepStandby(void)
{
  int Old_Value;
  lcdled_set(0);
  LCD_EnterSleep ();

  LCDBUS_Send(COMMAND, CMD_ENTRY_MODE); // deep standby mode
  Old_Value = LCDBUS_Read ();
  LCDBUS_Send(COMMAND, CMD_ENTRY_MODE); // deep standby mode
  LCDBUS_Send(DATA, Old_Value | 0x08); // DSTB (bit3) = 1
}

void LCD_ExitDeepStandby(void)
{
  int i;
  for (i=0; i<6; i++)
  {
    GPIO_ResetBits(GPIOD, GPIO_Pin_7);
    Delay (50);
    GPIO_SetBits(GPIOD, GPIO_Pin_7);
    Delay (50);
  }
  LCD_Init ();
}

void LCD_SetBacklight(int st)
{
  lcdled_set(st);
}

void LCD_Clear(u16 bgcolor)
{
  LCDBUS_Send (COMMAND, CMD_COLUMN_ADDRESS_SET); // Column address set
  LCDBUS_Send (DATA, 0);
  LCDBUS_Send (DATA, 0);

  LCDBUS_Send (COMMAND, CMD_PAGE_ADDRESS_SET); // Page address set
  LCDBUS_Send (DATA, 0);
  LCDBUS_Send (DATA, 0);
  
  LCDBUS_Send (COMMAND, CMD_MEMORY_WRITE); // memory write
  for (int i=0; i<76800; i++) // 320*240 = 76800
    //LCDBUS_Send (DATA, bgcolor);
    *LCDBUS_RSHIGH_ADDR = bgcolor;  
}


void LCD_DrawChar(unsigned int row, unsigned int col, char c, u16 bgcolor, u16 fgcolor)
{
  unsigned int x,y;
  int i,j;
  uc16 *font_char;
  
  x = 16 * col;
  if(x > (240-16))
    x = 240-16;
  y = 24 * row;
    if(y > (320-24))
  y = 320-24;
  
  c -= 32;
  if(c<=0)
    c = 0;
  if(c>94)
    c = 94;
  
  font_char = &ASCII_Table[c*24];
  for(i=0; i<24; i++)
  {
    for(j=0; j<16; ++j)
    {
      if(*font_char & (1u<<j))
        LCD_SetPixel (x+j, y+i, fgcolor);
      else
        LCD_SetPixel (x+j, y+i, bgcolor);
    }
    font_char++;
  }
}

#define SWAP_INT(A,B)	do {int __x; __x = (A); (A) = (B); (B) = __x;} while(0)

void LCD_DrawLine_norm(int x0, int y0, int x1, int y1, u16 color, bool swap)
{
  const int mul = (1<<12);
  int a, b, y;

  assert(x0 < x1);
  assert(y0 != y1);
  
  a = ((y1 - y0) * mul) / (x1 - x0);
  b = ((y0 * x1 - y1 * x0) * mul) / (x1 - x0);

  while(x0 <= x1)
  {
    y = a * x0 + b;
    y = (y + mul/2) / mul;
    if(!swap)
    {
      LCD_SetPixel(x0, y, color);
    }
    else
    {
      LCD_SetPixel(y, x0, color);
    }
    x0++;
  }
}

void LCD_DrawLine(int x0, int y0, int x1, int y1, u16 color)
{
  if(x0 == x1)
  {
    LCD_SetPixel(x0, y0, color);
    while(y0 != y1)
    {
      if(y0<y1)
        y0++;
      else
        y0--;
      LCD_SetPixel(x0, y0, color);
    }
    return;
  }

  if(y0 == y1)
  {
    LCD_SetPixel(x0, y0, color);
    while(x0 != x1)
    {
      if(x0<x1)
        x0++;
      else
        x0--;
      LCD_SetPixel(x0, y0, color);
    }
    return;
  }

  if(labs(x1-x0) > labs(y1-y0))
  {
    if(x0 > x1)
    {
      SWAP_INT(x0, x1);
      SWAP_INT(y0, y1);
    }
    LCD_DrawLine_norm(x0,y0,x1,y1, color, FALSE);
  }
  else
  {
    if(y0 > y1)
    {
      SWAP_INT(x0, x1);
      SWAP_INT(y0, y1);
    }
    LCD_DrawLine_norm(y0,x0,y1,x1, color, TRUE);
  }
}


void LCD_DrawCircle(int x, int y, int r, u16 color)
{
  int i, j;

  for(i=0; i<=r; i++)
  {
    j = (int)sqrt(r*r - i*i);
    LCD_SetPixel(x+i, y+j, color);
    LCD_SetPixel(x-i, y+j, color);
    LCD_SetPixel(x+i, y-j, color);
    LCD_SetPixel(x-i, y-j, color);
  }

  for(j=0; j<=r; j++)
  {
    i = (int)sqrt(r*r - j*j);
    LCD_SetPixel(x+i, y+j, color);
    LCD_SetPixel(x-i, y+j, color);
    LCD_SetPixel(x+i, y-j, color);
    LCD_SetPixel(x-i, y-j, color);
  }
}

#ifndef M_PI
  #define M_PI		3.14159265358979323846
#endif

void LCD_RotatePoint(int *px, int *py, int x, int y, int rot_deg)
{
  int rel_px = *px - x;
  int rel_py = *py - y;
  float polar_angle, polar_r;

  polar_r = sqrt(rel_px*rel_px + rel_py*rel_py);
  polar_angle = asin((float)rel_py / polar_r);
  if(rel_px < 0)
  {
    if(polar_angle >= 0.0)
    {
      polar_angle += M_PI/2;
    }
    else
    {
      polar_angle -= M_PI/2;
    }
  }

  polar_angle += ((float)rot_deg * M_PI) / 180.0;
  
  *px = (int)floor(cos(polar_angle) * polar_r) + x;
  *py = (int)floor(sin(polar_angle) * polar_r) + y;
}

void LCD_DrawRectBorder(int x, int y, int w, int h, u16 color)
{
  LCD_DrawLine (x, y, x+w, y, color);
  LCD_DrawLine (x+w, y, x+w, y+h, color);
  LCD_DrawLine (x, y+h, x+w, y+h, color);
  LCD_DrawLine (x, y, x, y+h, color);
}

void LCD_DrawRect(int x, int y, int w, int h, u16 color)
{
  int i, j;
  for (i=x; i<x+w; i++)
    for (j=y; j<y+h; j++)
      LCD_SetPixel (i, j, color);
}


/*-------------------------------------------------------------------------*/
static u16 LCD_ConBgColor;
static u16 LCD_ConFgColor;
static unsigned int LCD_ConRow;
static unsigned int LCD_ConCol;

void LCD_ConSetColor(u16 bgcolor, u16 fgcolor)
{
  LCD_ConBgColor = bgcolor;
  LCD_ConFgColor = fgcolor;
}


void LCD_ConSetPos(unsigned int row, unsigned int col)
{
  LCD_ConRow = row;
  LCD_ConCol = col;
}


void LCD_Printf(const char *fmt, ...)
{
  static char buf[256];
  char *p;
  va_list lst;
  
  va_start(lst, fmt);
  vsprintf(buf, fmt, lst);
  va_end(lst);
  
  p = buf;
  while(*p)
  {
    if(*p == '\n')
    {
      while(LCD_ConCol < 15)
      {
        LCD_DrawChar(LCD_ConRow, LCD_ConCol++, ' ',
        LCD_ConBgColor, LCD_ConFgColor);
      }
      LCD_ConRow++;
      LCD_ConCol = 0;
    }
    else
    {
      if(LCD_ConCol >= 15)
      {
        LCD_ConRow++;
        LCD_ConCol = 0;
      }
      LCD_DrawChar(LCD_ConRow, LCD_ConCol++, *p,
      LCD_ConBgColor, LCD_ConFgColor);
    }
    p++;
  }
}

void LCD_DrawPaletteH ()
{
  LCD_DrawRect (0,   0, 240, 40, LCD_COLOR_WHITE);
  LCD_DrawRect (0,  40, 240, 40, LCD_COLOR_RED);
  LCD_DrawRect (0,  80, 240, 40, LCD_COLOR_YELLOW);
  LCD_DrawRect (0, 120, 240, 40, LCD_COLOR_GREEN);
  LCD_DrawRect (0, 160, 240, 40, LCD_COLOR_CYAN);
  LCD_DrawRect (0, 200, 240, 40, LCD_COLOR_BLUE);
  LCD_DrawRect (0, 240, 240, 40, LCD_COLOR_MAGENTA);
  LCD_DrawRect (0, 280, 240, 40, LCD_COLOR_BLACK);
}

void LCD_DrawPaletteV ()
{
  LCD_DrawRect (  0, 0, 30, 320, LCD_COLOR_WHITE);
  LCD_DrawRect ( 30, 0, 30, 320, LCD_COLOR_RED);
  LCD_DrawRect ( 60, 0, 30, 320, LCD_COLOR_YELLOW);
  LCD_DrawRect ( 90, 0, 30, 320, LCD_COLOR_GREEN);
  LCD_DrawRect (120, 0, 30, 320, LCD_COLOR_CYAN);
  LCD_DrawRect (150, 0, 30, 320, LCD_COLOR_BLUE);
  LCD_DrawRect (180, 0, 30, 320, LCD_COLOR_MAGENTA);
  LCD_DrawRect (210, 0, 30, 320, LCD_COLOR_BLACK);
}
