/****************************************************************************
 *
 * Project: -----
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: lcd-320x240_9341.h
 * Description: Driver for TFT display with ILI9341 controller
 *
 ****************************************************************************/

#ifndef LCD_320X240_9341_H
#define LCD_320X240_9341_H

#include "stm32f10x_lib.h"

#ifdef __cplusplus
extern "C" {
#endif

//#define CHECK_ID

#define COMMAND 0
#define DATA    1

// LEVEL 1 COMMANDS
#define CMD_NOP 0x00
#define CMD_SOFTWARE_RESET 0x01
#define CMD_READ_ID 0x04
#define CMD_READ_STATUS 0x09
#define CMD_READ_POWER_MODE 0x0A
#define CMD_READ_MADCTL 0x0B
#define CMD_READ_PIXEL_FORMAT 0x0C
#define CMD_READ_IMAGE_FORMAT 0x0D
#define CMD_READ_SIGNAL_MODE 0x0E
#define CMD_READ_SELF_DIAGNOSTIC 0x0F
#define CMD_ENTER_SLEEP_MODE 0x10
#define CMD_SLEEP_OUT 0x11
#define CMD_PARTIAL_MODE_ON 0x12
#define CMD_NORMAL_MODE_ON 0x13
#define CMD_INVERSION_OFF 0x20
#define CMD_INVERSION_ON 0x21
#define CMD_GAMMA_SET 0x26
#define CMD_DISPLAY_OFF 0x28
#define CMD_DISPLAY_ON 0x29
#define CMD_COLUMN_ADDRESS_SET 0x2A
#define CMD_PAGE_ADDRESS_SET 0x2B
#define CMD_MEMORY_WRITE 0x2C
#define CMD_COLOR_SET 0x2D
#define CMD_MEMORY_READ 0x2E
#define CMD_PARTIAL_AREA 0x30
#define CMD_VERTICAL_SCROLLING 0x33
#define CMD_TEARING_EFFECT_OFF 0x34
#define CMD_TEARING_EFFECT_ON 0x35
#define CMD_MEMORY_ACCESS_CONTROL 0x36
#define CMD_VERTICAL_SCROLLING_START 0x37
#define CMD_IDLE_MODE_OFF 0x38
#define CMD_IDLE_MODE_ON 0x39
#define CMD_COLOR_PIXEL_FORMAT 0x3A
#define CMD_WRITE_MEMORY_CONTINUE 0x3C
#define CMD_READ_MEMORY_CONTINUE 0x3E
#define CMD_SET_TEAR_SCANLINE 0x44
#define CMD_GET_SCANLINE 0x45
#define CMD_WRITE_DISPLAY_BRIGHTNESS 0x51
#define CMD_READ_DISPLAY_BRIGHTNESS 0x52
#define CMD_WRITE_CTRL 0x53
#define CMD_READ_CTRL 0x54
#define CMD_WRITE_CABC_CONTROL 0x55
#define CMD_READ_CABC_CONTROL 0x56
#define CMD_WRITE_CABC_MIN_BRIGHTNESS 0x5E
#define CMD_READ_CABC_MIN_BRIGHTNESS 0x5F
#define CMD_READ_ID1 0xDA
#define CMD_READ_ID2 0xDB
#define CMD_READ_ID3 0xDC
#define CMD_READ_ID4 0xD3
  
// LEVEL 2 COMMANDS
#define CMD_RGB_INTERFACE_SIGNAL_CONTROL 0xB0
#define CMD_FRAME_RATE_NORMAL 0xB1
#define CMD_FRAME_RATE_IDLE 0xB2
#define CMD_FRAME_RATE_PARTIAL 0xB3
#define CMD_DISPLAY_INVERSION 0xB4
#define CMD_BLANKING_PORCH 0xB5
#define CMD_DISPLAY_FUNCTION 0xB6
#define CMD_ENTRY_MODE 0xB7
#define CMD_BACKLIGHT_CONTROL1 0xB8
#define CMD_BACKLIGHT_CONTROL2 0xB9
#define CMD_BACKLIGHT_CONTROL3 0xBA
#define CMD_BACKLIGHT_CONTROL4 0xBB
#define CMD_BACKLIGHT_CONTROL5 0xBC
//#define CMD_BACKLIGHT_CONTROL6 // "why is 5 afraid of 7? Because six seven ate", Master Joda
#define CMD_BACKLIGHT_CONTROL7 0xBE
#define CMD_BACKLIGHT_CONTROL8 0xBF
#define CMD_POWER1 0xC0
#define CMD_POWER2 0xC1
#define CMD_VCOM1 0xC5
#define CMD_VCOM2 0xC7
#define CMD_NV_MEMORY_WRITE 0xD0
#define CMD_NV_MEMORY_PROTECTION_KEY 0xD1
#define CMD_NV_MEMORY_STATUS 0xD2
#define CMD_READ_ID4 0xD3
#define CMD_POSITIVE_GAMMA 0xE0
#define CMD_NEGATIVE_GAMMA 0xE1
#define CMD_DIGITAL_GAMMA1 0xE2
#define CMD_DIGITAL_GAMMA2 0xE3
#define CMD_INTERFACE_CONTROL 0xF6


// COLORS in 16 bit format
#define LCD_COLOR_GRAY        0x8410
#define LCD_COLOR_LIGHTRED    0xfc10
#define LCD_COLOR_BLACK       0x0000
#define LCD_COLOR_NAVY        0x000F
#define LCD_COLOR_DARKGREEN   0x03E0
#define LCD_COLOR_DARKCYAN    0x03EF
#define LCD_COLOR_MAROON      0x7800
#define LCD_COLOR_PURPLE      0x780F
#define LCD_COLOR_OLIVE       0x7BE0
#define LCD_COLOR_LIGHTGREY   0xC618
#define LCD_COLOR_DARKGREY    0x7BEF
#define LCD_COLOR_BLUE        0x001F
#define LCD_COLOR_GREEN       0x07E0
#define LCD_COLOR_CYAN        0x07FF
#define LCD_COLOR_RED         0xF800
#define LCD_COLOR_MAGENTA     0xF81F
#define LCD_COLOR_YELLOW      0xFFE0
#define LCD_COLOR_WHITE       0xFFFF
#define LCD_COLOR_ORANGE      0xFD20
#define LCD_COLOR_GREENYELLOW 0xAFE5
#define LCD_COLOR_PINK        0xFC18

/* Form an LCD 16-bit color out of the given 8-bit Red/Green/Blue components. */
#define LCD_RGB_TO_LCDCOLOR(R,G,B)	\
	( (((R)>>3)&0x1f)<<11 | (((G)>>2)&0x3f)<<5 | (((B)>>3)&0x1f)<<0)

extern void LCD_Init(void);
extern int LCD_ReadID ();
extern void LCD_EnterSleep(void);
extern void LCD_ExitSleep(void);
extern void LCD_EnterDeepStandby(void);
extern void LCD_SetBacklight(int st);
extern void LCD_SetTestPattern(void);
extern void LCD_SetRandomPattern(void);
extern void LCD_Clear(u16 bgcolor);

extern void LCD_SetCursor(unsigned int x, unsigned int y);
extern void LCD_WriteRAM(u16 color);

extern void LCD_SetPixel(unsigned int x, unsigned int y, u16 color);
extern void LCD_WriteMem(unsigned int addr, const u16 *data, unsigned int nitems);
extern void LCD_FillMem(unsigned int addr, const u16 data, unsigned int nitems);

extern void LCD_DrawChar(unsigned int row, unsigned int col, char c, u16 bgcolor, u16 fgcolor);
extern void LCD_DrawLine(int x0, int y0, int x1, int y1, u16 color);
extern void LCD_DrawLine_norm(int x0, int y0, int x1, int y1, u16 color, bool swap);
extern void LCD_DrawCircle(int x, int y, int r, u16 color);
extern void LCD_RotatePoint(int *px, int *py, int x, int y, int rot_deg);
extern void LCD_DrawRect(int x, int y, int w, int h, u16 color);
extern void LCD_DrawRectBorder(int x, int y, int w, int h, u16 color);

extern void LCD_ConSetColor(u16 bgcolor, u16 fgcolor);
extern void LCD_ConSetPos(unsigned int row, unsigned int col);
extern void LCD_Printf(const char *fmt, ...);

extern void LCD_DrawPaletteH ();
extern void LCD_DrawPaletteV ();

#ifdef __cplusplus
}
#endif

#endif	/* LCD_320X240_H */


