/*************************************************************************
 *
*    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2007
 *
 *    File name   : smb380_drv.c
 *    Description : SMB380 acceleration sensor driver (I2C data mode)
 *
 *    History :
 *    1. Date        : 13, February 2008
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *
 *    $Revision: 18137 $
 **************************************************************************/
#include "includes.h"
// SPP +
// page 575

Boolean I2C_MasterWrite (Int8U SlaveAddr, pInt8U pData, Int32U Size)
{
  return I2C2_DataTransfer ((SlaveAddr<<1) | 0, pData, Size);
}

Boolean I2C_MasterRead (Int8U SlaveAddr, pInt8U pData, Int32U Size)
{
  return I2C2_DataTransfer ((SlaveAddr<<1) | 1, pData, Size);
}
// SPP -

/*************************************************************************
 * Function Name: SMB380_Init
 * Parameters: none
 *
 * Return: SMB380_Status_t
 *
 * Description: SMB380 init
 *
 *************************************************************************/
SMB380_Status_t SMB380_Init(void)
{
unsigned char Data[2];

  
  //Init I2C module as master
  // SPP +
  //I2C_InitMaster(I2C_SPEED);
  GPIO_InitTypeDef GPIO_InitStructure;
  // Enable GPIOE port
  RCC_APB2PeriphResetCmd(RCC_APB2Periph_GPIOE , DISABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOE , ENABLE);
  // Init Accl ready signal
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;   // SPP
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOE, &GPIO_InitStructure);
  I2C2_Init ();

  GPIOE->CRL = (GPIOE->CRL & 0xFFF0FFFF) | (0x00080000);  // PE4 - input, pull up/down; page 101 of datasheet
//  GPIOE->ODR = GPIOE->ODR | 0x10;                         // Pull-up  page 96
  // SPP -
    
  Data[0] = 0x14;

  I2C_MasterWrite(SMB380_ADDR, &Data[0], 1);
  
  I2C_MasterRead(SMB380_ADDR, &Data[1], 1);
  
  Data[1] &= ~(7<<0);

  I2C_MasterWrite(SMB380_ADDR, &Data[0], 2);
  
  Data[0] = 0x15;

  I2C_MasterWrite(SMB380_ADDR, &Data[0], 1);
  
  I2C_MasterRead(SMB380_ADDR, &Data[1], 1);
  
  Data[1] |= ((1<<5) | (2<<1) | (1<<0));
    
  I2C_MasterWrite(SMB380_ADDR, &Data[0], 2);

  return SMB380_PASS;
}
/*************************************************************************
 * Function Name: SMB380_GetID
 * Parameters: none
 *
 * Return: SMB380_Status_t
 *
 * Description: SMB380 get chip ID and revision
 *
 *************************************************************************/
SMB380_Status_t SMB380_GetID (pInt8U pChipId, pInt8U pRevision)
{
unsigned char buf[2] = {SMB380_CHIP_ID};
  //Write the address of Chip ID register
  I2C_MasterWrite(SMB380_ADDR, buf, 1);
  
  I2C_MasterRead(SMB380_ADDR, buf, 2);
  *pChipId = buf[0];
  *pRevision = buf[1];
  
  return SMB380_PASS;
}

/*************************************************************************
 * Function Name: SMB380_Init
 * Parameters: none
 *
 * Return: SMB380_Status_t
 *
 * Description: SMB380 init
 *
 *************************************************************************/
SMB380_Status_t SMB380_GetData (pSMB380_Data_t pData)
{
  unsigned char regaddr = SMB380_ACCX_ADDR;
  
  I2C_MasterWrite(SMB380_ADDR, &regaddr, 1);
  I2C_MasterRead(SMB380_ADDR, (unsigned char *)pData, sizeof(SMB380_Data_t));
  // least significant bit of each axis data shows if there are actual new data
  if (!((pData->AccX & 0x01) && (pData->AccY & 0x01) && (pData->AccZ & 0x01)))
    return SMB380_FAIL;
  // the REAL data is placed on the most significant 10 of the 16 bits (15 downto 6)
  // ==> least significant 6 (5 downto 0) are ignored.
  pData->AccX = pData->AccX >> 6;
  pData->AccY = pData->AccY >> 6;
  pData->AccZ = pData->AccZ >> 6;
  return SMB380_PASS;
}

SMB380_Status_t SMB380_IntClear (void)
{
unsigned char Data[2];
      
  Data[0] = 0x0A;

  I2C_MasterWrite(SMB380_ADDR, &Data[0], 1);
  
  I2C_MasterRead(SMB380_ADDR, &Data[1], 1);
  
  Data[1] |= (1<<6);

  I2C_MasterWrite(SMB380_ADDR, &Data[0], 2);
  
  return SMB380_PASS; // SPP
}


