/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2009
 *
 *    File name   : drv_i2c1.c
 *    Description : I2C2 Master mode driver
 *
 *    History :
 *    1. Date        : February, 2009
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: 29 $
 **************************************************************************/
#define I2C2_DRV_GLOBAL
#include "i2c2_drv.h"

static Boolean _I2C_NotUsed;
static volatile Boolean _Done;
static volatile Boolean _Error;
static volatile Boolean _Reinit;

static Int8U _SlaveAddr;
static Int32U _Size;
static pInt8U _pData;

#define I2C_DLY()     for (volatile Int32U i2c_dly_cntr = 7; i2c_dly_cntr; i2c_dly_cntr--);

/*************************************************************************
 * Function Name: I2C2_ErrIntrHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: I2C2 error interrupt handler
 *
 *************************************************************************/
void I2C2_ErrIntrHandler(void)
{
Int32U flags;
  flags   = I2C2->SR1;
  flags  |= I2C2->SR2 << 16;
  flags  &= 0x00FFFFFF;

  if (I2C_SR1_AF & flags)
  {
    // Generate Stop condition (return back to slave mode)
    I2C2->CR1 |= I2C_CR1_STOP;
  }
  // something is wrong, reinit I2C
  NVIC->ICPR[1] = 1UL << 0;
  NVIC->ICER[1] = 1UL << 0;
  _Reinit = TRUE;
  _Done   = TRUE;
  _Error  = TRUE;
}

/*************************************************************************
 * Function Name: I2C2_EvnIntrHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: I2C2 event interrupt handler
 *
 *************************************************************************/
Int32U flagsl_hold;
Int32U flagsh_hold;
void I2C2_EvnIntrHandler(void)
{
Int32U flagsl;
Int32U flagsh;
  flagsl = I2C2->SR1;
  flagsh = I2C2->SR2;
  flagsl_hold = flagsl;
  flagsh_hold = flagsh;
  if ((I2C_SR2_MSL | I2C_SR2_TRA) == ((I2C_SR2_MSL | I2C_SR2_TRA) & flagsh))
  {
    // Master transmite mode
    if (I2C_SR1_ADDR & flagsl)
    {
      // do nothing
    }
    else if (I2C_SR1_TXE & flagsl)
    {
      if(0 == _Size)
      {
        _Done = TRUE;
      }
      else
      {
        --_Size;
        I2C2->DR = *_pData++;

        if(0 == _Size)
        {
          while(0 == (I2C2->SR1 & I2C_SR1_BTF));
          I2C2->CR1 |= I2C_CR1_STOP;
        }
      }
    }
    else
    {
      // something is wrong,  reinit I2C
      _Done   = TRUE;
      _Error  = TRUE;
      _Reinit = TRUE;
      NVIC->ICPR[0] = 1UL << 31;
      NVIC->ICER[0] = 1UL << 31;
    }
  }
  else if ((I2C_SR2_MSL) == ((I2C_SR2_MSL | I2C_SR2_TRA) & flagsh))
  {
    // Master receive or start condition done
    if ((I2C_SR1_SB) & flagsl)
    {
      I2C2->DR = _SlaveAddr;
      if(0 == _Size)
      {
        I2C2->CR1 |= I2C_CR1_STOP;
        _Done = TRUE;
      }
    }
    else if (I2C_SR1_ADDR & flagsl)
    {
      if (1 == _Size)
      {
        I2C2->CR1 &=~I2C_CR1_ACK;
        I2C2->CR1 |= I2C_CR1_STOP;
      }
    }
    else if (I2C_SR1_RXNE & flagsl)
    {
      if (1 == --_Size)
      {
        I2C2->CR1 &=~I2C_CR1_ACK;
        I2C2->CR1 |= I2C_CR1_STOP;
      }

      *_pData++ = I2C2->DR;
      if (0 == _Size)
      {
        _Done = TRUE;
      }
    }
    else
    {
      // something is wrong,  reinit I2C
      _Done   = TRUE;
      _Error  = TRUE;
      _Reinit = TRUE;
      NVIC->ICPR[0] = 1UL << 31;
      NVIC->ICER[0] = 1UL << 31;
    }
  }
  else
  {
    if (I2C_SR1_STOPF & I2C2->SR1 )
    {
      // something is wrong, reinit I2C
      NVIC->ICPR[0] = 1UL << 31;
      NVIC->ICER[0] = 1UL << 31;
      _Reinit = TRUE;
      _Done  = TRUE;
      _Error = TRUE;
    }
    I2C2->SR1 = 0;
  }
}
static void my_delay(void){
	for(volatile int i=0; i<10000; i++);
}
static void generate_scl_toggles(int n)
{
	while(n--) {
		GPIO_InitTypeDef GPIO_InitStructure;
		// set zero output 
		GPIO_InitStructure.GPIO_Pin = (1<<10);
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
		GPIO_Init(GPIOB, &GPIO_InitStructure);
		GPIO_ResetBits(GPIOB, (1<<10));
		my_delay();
		// set input
		GPIO_InitStructure.GPIO_Pin = (1<<10);
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_Init(GPIOB, &GPIO_InitStructure);
		// set input
		GPIO_InitStructure.GPIO_Pin = (1<<11);
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_Init(GPIOB, &GPIO_InitStructure);
		my_delay();
	}
}


/*************************************************************************
 * Function Name: I2C2_Init
 * Parameters: none
 *
 * Return: none
 *
 * Description: Init I2C2 interface
 *
 *************************************************************************/
void I2C2_Init (void)
{
I2C_InitTypeDef  I2C_InitStructure;
GPIO_InitTypeDef GPIO_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;

  _I2C_NotUsed = TRUE;

  // Enable clock
  I2C_DeInit(I2C2);
  
  generate_scl_toggles(10);
  
  RCC_APB1PeriphClockCmd(  RCC_APB1Periph_I2C2,
                           ENABLE);
  RCC_APB2PeriphClockCmd(  RCC_APB2Periph_GPIOB,
                           ENABLE);

  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_10 | GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;  // SPP
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  // I2C configuration
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
  I2C_InitStructure.I2C_OwnAddress1 = 0xAA;
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
  I2C_InitStructure.I2C_ClockSpeed = I2C2_SPEED;

  // I2C Peripheral Enable
  I2C_Cmd(I2C2, ENABLE);

  // Apply I2C configuration after enabling it
  I2C_Init(I2C2, &I2C_InitStructure);

  // Enable the I2C2 Events Interrupt
  NVIC_InitStructure.NVIC_IRQChannel = I2C2_EV_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = I2C2_INTR_PRIO;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = I2C2_INTR_SUBPRIO;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  // Enable the I2C2 Errors Interrupt
  NVIC_InitStructure.NVIC_IRQChannel = I2C2_ER_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = I2C2_INTR_PRIO;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = I2C2_INTR_SUBPRIO;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  // Enable interrupts from I2C2 module
  I2C_ITConfig(I2C2, I2C_IT_BUF | I2C_IT_EVT | I2C_IT_ERR, ENABLE);
}

/*************************************************************************
 * Function Name: I2C2_Open
 * Parameters: none
 *
 * Return: Boolean
 *
 * Description: Init I2C interface open
 *
 *************************************************************************/
Boolean I2C2_Open (void)
{
  return(0 != AtomicExchange(FALSE, &_I2C_NotUsed));
}

/*************************************************************************
 * Function Name: I2C2_Close
 * Parameters: none
 *
 * Return: none
 *
 * Description: Init I2C2 interface release
 *
 *************************************************************************/
void I2C2_Close (void)
{
  AtomicExchange(TRUE, &_I2C_NotUsed);
}

/*************************************************************************
 * Function Name: I2C2_DataTransfer
 * Parameters: Int8U SlaveAddr, pInt8U pData, Int32U Size
 *
 * Return: none
 *
 * Description: I2C transfer data to/from slave
 *
 *************************************************************************/
Boolean I2C2_DataTransfer (Int8U SlaveAddr, pInt8U pData, Int32U Size)
{
  _Error = FALSE;
  _Done  = FALSE;
  _SlaveAddr = SlaveAddr;
  _Size = Size;
  _pData = pData;

  I2C2->CR1 |= I2C_CR1_ACK | I2C_CR1_START;
  Int32U TO = 100000;
  while(FALSE == _Done)
  {
    if(0 == --TO)
    {
      _Reinit = TRUE;
      break;
    }
  }
  if (_Reinit)
  {
      _Reinit = FALSE;
      I2C2_Init();
  }
  return(!_Error && _Done);
}
